pure = {}
pure._files = {}
pure._current = {}
pure.approved = {}
pure.enabled = false
pure.verbosity = 0 -- Different levels of logging

local function exists(file)
    local fh = io.open(file)
    local e = not not fh
    if fh then fh:close() end
    return e
end

-- Recursive search-and-retrieve
local function deepFind(dir, file)
    local found = {}
    for f in io.enumdir(dir, true) do -- Exclude . and ..
        if (io.isdir(dir.."/"..f)) then
            local res = deepFind(dir.."/"..f, file)
            for _, v in pairs(res) do
                table.insert(found, dir.."/"..f.."/"..v)
            end
        else
            if (f:find(file)) then
                table.insert(found, dir.."/"..f)
            end
        end
    end
    return found
end

function pure.getSum(file)
    for k, v in pairs(pure._files) do
        if (v[1] == file) then
            return v[2]
        end
    end
    return nil
end

function pure.addFile(file, sum)
    file = file:gsub("#(.+)", "")
    if (file == "") then return end

    if (file:find("*")) then
        local dir
        file:gsub("(.+)/", function(a) dir = a end)

        local found = deepFind(dir, file:sub(#dir+2):gsub("*", ".+"))
        if (#found == 0) then
            if (pure.verbosity > 1) then print('\169115255110Pure [2]: No file has been found matching "'..file..'"') end
            return
        end

        for _, v in pairs(found) do
            table.insert(pure._files, {v, sum or checksumfile(v)})  -- If for some reason you want to specify one checksum for a file range you can
        end
    else
        if (not exists(file)) then
            if (pure.verbosity > 1) then print('\169115255110Pure [2]: No file has been found matching "'..file..'"') end
            return
        end
        table.insert(pure._files, {file, sum or checksumfile(file)})
    end
end

-- Parse config file, remove comments, process predefined SHA sums if given
do
    local fh = io.open("sys/pure.cfg", "r")
    if (fh == nil) then
        if (pure.verbosity > 0) then print("\169115255110Pure [1]: No pure.cfg found! Creating blank file...") end
        fh:close()
        fh = io.open("sys/pure.cfg", "w")
        fh:close()
    end

    for l in fh:lines() do
        if (l:sub(1,10) == "#verbosity") then
            pure.verbosity = tonumber(l:sub(12,12))
            if (pure.verbosity > 0) then print("\169115255110Pure [1]: Set verbosity level to " .. pure.verbosity .. ".") end
        else
            l:gsub("(.+)\t?([0-9a-f]*)%s*", function(a, b)
                if (b == '') then b = nil end
                pure.addFile(a, b)
            end)
        end
    end
    fh:close()
end

addhook("parse", "pure._cmd")
function pure._cmd(cmd)
    local n = cmd:find("sv_pure")
    if (n) then
        pure.enabled = (tonumber(cmd:sub(n+8, n+8)) == 1)
        if (pure.verbosity > -1) then print("\169115255110Pure: Pure mode "..(pure.enabled and "enabled" or "disabled").."!") end
        return 1
    end
end

addhook("join", "pure._join")
function pure._join(id)
    if (pure.enabled) then
        pure._current[id] = {pure._files[1], 1}
        reqcld(id, 4, pure._current[id][1][1])
        if (pure.verbosity > 2) then print('\169115255110Pure [3]: Requesting '..pure._current[id][1][1]..' from client ID #'..id) end
        timer(500, "parse", "lua \"reqcld("..id..", 4, [=["..pure._current[id][1][1].."]=])\"", -1) -- Re-request in case of failed delivery
        pure.approved[id] = false
    end
end

addhook("leave", "pure._leave")
function pure._leave(id)
    pure.approved[id] = false
    pure._current[id] = nil
end

addhook("clientdata", "pure._clientdata")
function pure._clientdata(id, m, a1, a2)
    if (m == 4) then
        if (pure.enabled and pure._current[id]) then
            freetimer("parse", "lua \"reqcld("..id..", 4, [=["..pure._current[id][1][1].."]=])\"")
            local sum = pure.getSum(pure._current[id][1][1])
            if (pure.verbosity > 3) then print('\169115255110Pure [4]: Client ID #'..id..' ('..pure._current[id][1][1]..'): "'..tostring(sum)..'" local, "'..a2..'" remote') end
            if (sum and a2 ~= sum) then
                parse('kick '..id..' "This server is running in pure mode."')
                return
            end

            pure._current[id][2] = pure._current[id][2] + 1
            pure._current[id][1] = pure._files[pure._current[id][2]]
            if (pure._current[id][1] == nil) then
                pure._current[id] = nil
                if (pure.verbosity > 0) then print('\169115255110Pure [1]: Client ID #'..id..' has passed all checks') end
                pure.approved[id] = true
            else
                -- C stack overflow prevention
                timer(1, "parse", "lua \"reqcld("..id..", 4, [=["..pure._current[id][1][1].."]=])\"")
                timer(500, "parse", "lua \"reqcld("..id..", 4, [=["..pure._current[id][1][1].."]=])\"", -1)
                if (pure.verbosity > 2) then print('\169115255110Pure [3]: Requesting '..pure._current[id][1][1]..' from client ID #'..id) end
            end
        end
    end
end